p, option, select, input, textarea {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 8pt;
  font-style: normal;
  font-variant: normal;
  font-weight: normal;
}

input[type="submit"], select {
  height: 14pt;
}

textarea {
  font-family: monospace;
  border: 0;
  margin: 0;
}

a.opener {
  text-decoration: none;
}

td.control img {
  border: none;
  margin: 2px;
  padding: 0;
}

td.control {
  width: 30px;
  vertical-align: middle;
  line-height: 1px;
  font-size: 1px;
}

a.opener img, h3 img {
  border: none;
  margin-top: 5px;
  margin-bottom: -5px;
  margin-left: 5px;
}

h2 {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 12pt;
  font-style: normal;
  font-variant: small-caps;
  font-weight: bold;
}

h1 {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 12pt;
  font-style: normal;
  font-weight: bold;
}

h2 img {
  border: none;
  margin-right: 10px;
}

h3 {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 10pt;
  font-style: normal;
  font-variant: normal;
  font-weight: bold;
  background-color: #cccccc;
  margin-bottom: 0;
  margin-top: 10px;
  padding-top: 2px;
  padding-bottom: 3px;
}

div.fetch {
  border-top: solid #999999 1px;
  width: 100%;
  background-color: #ddddcc;
  margin: 0;
}

div.fetch th {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 9pt;
  font-style: normal;
  font-variant: normal;
  font-weight: bold;
}

div.fetch td {
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-size: 9pt;
  font-style: normal;
  font-variant: normal;
  font-weight: normal;
}

.header {
  margin-left: 10pt;
}

.saver a {
  font-size: 9pt;
  font-weight: normal;
  text-decoration: none;
  color: blue;
}

.saver a:hover {
  color: red;
}

form {
  margin: 0;
  padding: 0;
}

td.tdcol1 {
  width: 100px;
}

td.tdcol2 input {
  width: 550px;
}

input:focus, textarea:focus, select:focus {
  background-color: #ffc;
}

textarea.fetch_ta {
  width: 100%;
  height: 100%;
  margin: 0;
  padding: 0;
}

textarea.comment_ta {
  width: 100%;
  height: 100%;
  font-family: Helvetica, Verdana, Arial, sans-serif;
  font-style: italic;
  font-size: 6.5pt;
  margin: 0;
  padding: 0;
}

td.comment_ta {
  width: 190px;
  overflow: hidden;
}

td.tdc1, td.tdc2, td.tdc3 {
  margin: 0;
  padding: 0px;
  overflow: hidden;
}

td.tdc1 {
  background-color: #aaddaa;
  background-image: url(tdc1-resize.gif);
  background-position: bottom center;
  background-repeat: no-repeat;
}

td.tdc2 {
  background-color: #ddaaaa;
  background-image: url(tdc2-resize.gif);
  background-position: bottom center;
  background-repeat: no-repeat;
}

td.tdc3 {
  background-color: #aaaadd;
  background-image: url(tdc3-resize.gif);
  background-position: bottom center;
  background-repeat: no-repeat;
}

tr.tdc {
  margin: 0;
  padding: 0;
  overflow: hidden;
}

table.tdc {
  width: 100%;
  overflow: hidden;
}

table.inner_tdc {
  width: 100%;
  height: 100%;
  margin: 0;
  padding: 0;
  border-collapse: separate;
  border-spacing: 2px;
  padding-bottom: 5px;
  overflow: hidden;
}

table.inner_tdc tr, table.inner_tdc td {
  margin: 0;
  padding: 0;
  height: 100%;
  overflow: hidden;
  vertical-align: text-top;
}

img.pfeil {
  padding-bottom: 4px;
}

div#progress, div#menucontainer {
  position: fixed;
  z-index: 100;
  right: 8px;
  top: 8px;
}

div#progress {
  width: 90px;
  height: 15px;
  border: ridge #996 2px;
  background-color: #996;
  overflow: hidden;
}

div#progress img {
  margin: 0;
  padding: 0;
}

div#menucontainer {
  border: ridge #996 2px;
  background-color: #eeb;
  overflow: hidden;
  text-align: right;
  padding: 0 2pt;
  cursor: pointer;
}

div#menu {
  border-top: solid #996 1px;
  padding-top: 2pt;
  cursor: default;
}

div#menu ul {
  list-style: none;
  margin: 0;
  padding: 0;
}

div#menu ul li a {
  text-decoration: none;
  color: #00f;
}

div#menu ul li a:hover {
  color: #f00;
}
